@@ -31,6 +31,7 @@ import (
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	"kubevirt.io/containerized-data-importer/pkg/common"
 	"kubevirt.io/containerized-data-importer/pkg/token"
+	"kubevirt.io/containerized-data-importer/pkg/util"
 	"kubevirt.io/containerized-data-importer/pkg/util/cert/fetcher"
 	"kubevirt.io/containerized-data-importer/pkg/util/cert/generator"
 )
@@ -86,6 +87,7 @@ type CloneReconciler struct {
 	image               string
 	verbose             string
 	pullPolicy          string
+	installerLabels     map[string]string
 }
 
 // NewCloneController creates a new instance of the config controller.
@@ -95,7 +97,8 @@ func NewCloneController(mgr manager.Manager,
 	verbose string,
 	clientCertGenerator generator.CertGenerator,
 	serverCAFetcher fetcher.CertBundleFetcher,
-	apiServerKey *rsa.PublicKey) (controller.Controller, error) {
+	apiServerKey *rsa.PublicKey,
+	installerLabels map[string]string) (controller.Controller, error) {
 	reconciler := &CloneReconciler{
 		client:              mgr.GetClient(),
 		scheme:              mgr.GetScheme(),
@@ -107,6 +110,7 @@ func NewCloneController(mgr manager.Manager,
 		recorder:            mgr.GetEventRecorderFor("clone-controller"),
 		clientCertGenerator: clientCertGenerator,
 		serverCAFetcher:     serverCAFetcher,
+		installerLabels:     installerLabels,
 	}
 	cloneController, err := controller.New("clone-controller", mgr, controller.Options{
 		Reconciler: reconciler,
@@ -185,8 +189,8 @@ func (r *CloneReconciler) Reconcile(_ context.Context, req reconcile.Request) (r
 			"NOT has annotation(AnnCloneOf)", !metav1.HasAnnotation(pvc.ObjectMeta, AnnCloneOf),
 			"isBound", isBound(pvc, log),
 			"has finalizer?", HasFinalizer(pvc, cloneSourcePodFinalizer))
-		if HasFinalizer(pvc, cloneSourcePodFinalizer) {
-			// Clone completed, remove source pod and finalizer.
+		if HasFinalizer(pvc, cloneSourcePodFinalizer) || pvc.DeletionTimestamp != nil {
+			// Clone completed, remove source pod and/or finalizer
 			if err := r.cleanup(pvc, log); err != nil {
 				return reconcile.Result{}, err
 			}
@@ -294,16 +298,7 @@ func (r *CloneReconciler) updatePvcFromPod(sourcePod *corev1.Pod, pvc *corev1.Pe
 		r.recorder.Event(pvc, corev1.EventTypeNormal, CloneSucceededPVC, cloneComplete)
 	}
 
-	if sourcePod != nil && sourcePod.Status.ContainerStatuses != nil {
-		// update pvc annotation tracking pod restarts only if the source pod restart count is greater
-		// see the same in upload-controller
-		annPodRestarts, _ := strconv.Atoi(pvc.Annotations[AnnPodRestarts])
-		podRestarts := int(sourcePod.Status.ContainerStatuses[0].RestartCount)
-		if podRestarts > annPodRestarts {
-			pvc.Annotations[AnnPodRestarts] = strconv.Itoa(podRestarts)
-		}
-		setConditionFromPodWithPrefix(pvc.Annotations, AnnSourceRunningCondition, sourcePod)
-	}
+	setAnnotationsFromPodWithPrefix(pvc.Annotations, sourcePod, AnnSourceRunningCondition)
 
 	if !reflect.DeepEqual(currentPvcCopy, pvc) {
 		return r.updatePVC(pvc)
@@ -420,10 +415,12 @@ func (r *CloneReconciler) cleanup(pvc *corev1.PersistentVolumeClaim, log logr.Lo
 			log.V(3).Info("Clone succeeded, waiting for source pod to stop running", "pod.Namespace", pod.Namespace, "pod.Name", pod.Name)
 			return nil
 		}
-
-		if err = r.client.Delete(context.TODO(), pod); err != nil {
-			if !k8serrors.IsNotFound(err) {
-				return errors.Wrap(err, "error deleting clone source pod")
+		if shouldDeletePod(pvc) {
+			log.V(3).Info("Deleting pod", "pod.Name", pod.Name)
+			if err = r.client.Delete(context.TODO(), pod); err != nil {
+				if !k8serrors.IsNotFound(err) {
+					return errors.Wrap(err, "error deleting clone source pod")
+				}
 			}
 		}
 	}
@@ -478,6 +475,7 @@ func (r *CloneReconciler) CreateCloneSourcePod(image, pullPolicy, clientName str
 	}
 
 	pod := MakeCloneSourcePodSpec(sourceVolumeMode, image, pullPolicy, sourcePvcName, sourcePvcNamespace, ownerKey, clientKey, clientCert, serverCABundle, pvc, podResourceRequirements, workloadNodePlacement)
+	util.SetRecommendedLabels(pod, r.installerLabels, "cdi-controller")
 
 	if err := r.client.Create(context.TODO(), pod); err != nil {
 		return nil, errors.Wrap(err, "source pod API create errored")
