@@ -52,6 +52,7 @@ type CDIConfigReconciler struct {
 	uploadProxyServiceName string
 	configName             string
 	cdiNamespace           string
+	installerLabels        map[string]string
 }
 
 func isErrCacheNotStarted(err error) bool {
@@ -113,6 +114,8 @@ func (r *CDIConfigReconciler) Reconcile(_ context.Context, req reconcile.Request
 }
 
 func (r *CDIConfigReconciler) setOperatorParams(config *cdiv1.CDIConfig) error {
+	util.SetRecommendedLabels(config, r.installerLabels, "cdi-controller")
+
 	cdiCR, err := GetActiveCDI(r.client)
 	if err != nil {
 		return err
@@ -333,6 +336,7 @@ func (r *CDIConfigReconciler) createCDIConfig() (*cdiv1.CDIConfig, error) {
 			if err := operator.SetOwnerRuntime(r.uncachedClient, config); err != nil {
 				return nil, err
 			}
+			util.SetRecommendedLabels(config, r.installerLabels, "cdi-controller")
 			if err := r.client.Create(context.TODO(), config); err != nil {
 				if errors.IsAlreadyExists(err) {
 					config := &cdiv1.CDIConfig{}
@@ -351,7 +355,6 @@ func (r *CDIConfigReconciler) createCDIConfig() (*cdiv1.CDIConfig, error) {
 }
 
 func (r *CDIConfigReconciler) reconcileImportProxy(config *cdiv1.CDIConfig) error {
-	log := r.log.WithName("CDIconfig").WithName("ImportProxyReconcile")
 	config.Status.ImportProxy = config.Spec.ImportProxy
 
 	// Avoid nil pointers and segfaults for the initial case, where ImportProxy is nil for both the spec and the status.
@@ -366,15 +369,17 @@ func (r *CDIConfigReconciler) reconcileImportProxy(config *cdiv1.CDIConfig) erro
 		// Try Openshift cluster wide proxy only if the CDIConfig default config is empty
 		clusterWideProxy, err := GetClusterWideProxy(r.client)
 		if err != nil {
-			log.V(3).Info(err.Error())
-			return nil
+			return err
 		}
 		config.Status.ImportProxy.HTTPProxy = &clusterWideProxy.Status.HTTPProxy
 		config.Status.ImportProxy.HTTPSProxy = &clusterWideProxy.Status.HTTPSProxy
 		config.Status.ImportProxy.NoProxy = &clusterWideProxy.Status.NoProxy
 		if clusterWideProxy.Spec.TrustedCA.Name != "" {
 			config.Status.ImportProxy.TrustedCAProxy = &clusterWideProxy.Spec.TrustedCA.Name
-			r.reconcileImportProxyCAConfigMap(config, clusterWideProxy)
+			err = r.reconcileImportProxyCAConfigMap(config, clusterWideProxy)
+			if err != nil {
+				return err
+			}
 		}
 	}
 	return nil
@@ -382,25 +387,29 @@ func (r *CDIConfigReconciler) reconcileImportProxy(config *cdiv1.CDIConfig) erro
 
 // Create/Update a configmap with the CA certificates in the controllor context with the cluster-wide proxy CA certificates to be used by the importer pod
 func (r *CDIConfigReconciler) reconcileImportProxyCAConfigMap(config *cdiv1.CDIConfig, proxy *ocpconfigv1.Proxy) error {
+	client := r.uncachedClient
 	cmName := proxy.Spec.TrustedCA.Name
 	if cmName == "" {
 		// Using the default cluster-wide proxy CA certificates configmap name
 		cmName = ClusterWideProxyConfigMapName
 	}
 	clusterWideProxyConfigMap := &v1.ConfigMap{}
-	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: cmName, Namespace: ClusterWideProxyConfigMapNameSpace}, clusterWideProxyConfigMap); err == nil {
+	if err := client.Get(context.TODO(), types.NamespacedName{Name: cmName, Namespace: ClusterWideProxyConfigMapNameSpace}, clusterWideProxyConfigMap); err == nil {
 		// Copy the cluster-wide proxy CA certificates to the importer pod proxy CA certificates configMap
 		if certBytes, ok := clusterWideProxyConfigMap.Data[ClusterWideProxyConfigMapKey]; ok {
 			configMap := &v1.ConfigMap{}
-			if err := r.client.Get(context.TODO(), types.NamespacedName{Name: common.ImportProxyConfigMapName, Namespace: r.cdiNamespace}, configMap); errors.IsNotFound(err) {
-				if err := r.client.Create(context.TODO(), r.createProxyConfigMap(certBytes)); err != nil {
+			if err := client.Get(context.TODO(), types.NamespacedName{Name: common.ImportProxyConfigMapName, Namespace: r.cdiNamespace}, configMap); errors.IsNotFound(err) {
+				proxyConfigMap := r.createProxyConfigMap(certBytes)
+				util.SetRecommendedLabels(proxyConfigMap, r.installerLabels, "cdi-controller")
+				if err := client.Create(context.TODO(), proxyConfigMap); err != nil {
 					return err
 				}
 				return nil
 			}
 			if configMap != nil {
 				configMap.Data[common.ImportProxyConfigMapKey] = certBytes
-				if err := r.client.Update(context.TODO(), configMap); err != nil {
+				util.SetRecommendedLabels(configMap, r.installerLabels, "cdi-controller")
+				if err := client.Update(context.TODO(), configMap); err != nil {
 					return err
 				}
 			}
@@ -425,7 +434,7 @@ func (r *CDIConfigReconciler) Init() error {
 }
 
 // NewConfigController creates a new instance of the config controller.
-func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServiceName, configName string) (controller.Controller, error) {
+func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServiceName, configName string, installerLabels map[string]string) (controller.Controller, error) {
 	uncachedClient, err := client.New(mgr.GetConfig(), client.Options{
 		Scheme: mgr.GetScheme(),
 		Mapper: mgr.GetRESTMapper(),
@@ -441,6 +450,7 @@ func NewConfigController(mgr manager.Manager, log logr.Logger, uploadProxyServic
 		uploadProxyServiceName: uploadProxyServiceName,
 		configName:             configName,
 		cdiNamespace:           util.GetNamespace(),
+		installerLabels:        installerLabels,
 	}
 
 	configController, err := controller.New("config-controller", mgr, controller.Options{
@@ -596,7 +606,7 @@ func watchClusterProxy(mgr manager.Manager, configController controller.Controll
 }
 
 func getURLFromIngress(ing *networkingv1.Ingress, uploadProxyServiceName string) string {
-	if ing.Spec.DefaultBackend != nil {
+	if ing.Spec.DefaultBackend != nil && ing.Spec.DefaultBackend.Service != nil {
 		if ing.Spec.DefaultBackend.Service.Name != uploadProxyServiceName {
 			return ""
 		}
@@ -607,7 +617,7 @@ func getURLFromIngress(ing *networkingv1.Ingress, uploadProxyServiceName string)
 			continue
 		}
 		for _, path := range rule.HTTP.Paths {
-			if path.Backend.Service.Name == uploadProxyServiceName {
+			if path.Backend.Service != nil && path.Backend.Service.Name == uploadProxyServiceName {
 				if rule.Host != "" {
 					return rule.Host
 				}
@@ -615,7 +625,6 @@ func getURLFromIngress(ing *networkingv1.Ingress, uploadProxyServiceName string)
 		}
 	}
 	return ""
-
 }
 
 func getURLFromRoute(route *routev1.Route, uploadProxyServiceName string) string {
