@@ -2,11 +2,9 @@ package controller
 
 import (
 	"context"
-	"encoding/json"
 	"fmt"
 	"net/url"
 	"reflect"
-	"regexp"
 	"strconv"
 	"time"
 
@@ -95,10 +93,6 @@ const (
 	ImportTargetInUse = "ImportTargetInUse"
 )
 
-var (
-	vddkInfoMatch = regexp.MustCompile(`((.*; )|^)VDDK: (?P<info>{.*})`)
-)
-
 // ImportReconciler members
 type ImportReconciler struct {
 	client             client.Client
@@ -111,6 +105,7 @@ type ImportReconciler struct {
 	pullPolicy         string
 	filesystemOverhead string
 	featureGates       featuregates.FeatureGates
+	installerLabels    map[string]string
 }
 
 type importPodEnvVar struct {
@@ -137,22 +132,23 @@ type importPodEnvVar struct {
 }
 
 // NewImportController creates a new instance of the import controller.
-func NewImportController(mgr manager.Manager, log logr.Logger, importerImage, pullPolicy, verbose string) (controller.Controller, error) {
+func NewImportController(mgr manager.Manager, log logr.Logger, importerImage, pullPolicy, verbose string, installerLabels map[string]string) (controller.Controller, error) {
 	uncachedClient, err := client.New(mgr.GetConfig(), client.Options{
 		Scheme: mgr.GetScheme(),
 		Mapper: mgr.GetRESTMapper(),
 	})
 	client := mgr.GetClient()
 	reconciler := &ImportReconciler{
-		client:         client,
-		uncachedClient: uncachedClient,
-		scheme:         mgr.GetScheme(),
-		log:            log.WithName("import-controller"),
-		image:          importerImage,
-		verbose:        verbose,
-		pullPolicy:     pullPolicy,
-		recorder:       mgr.GetEventRecorderFor("import-controller"),
-		featureGates:   featuregates.NewFeatureGates(client),
+		client:          client,
+		uncachedClient:  uncachedClient,
+		scheme:          mgr.GetScheme(),
+		log:             log.WithName("import-controller"),
+		image:           importerImage,
+		verbose:         verbose,
+		pullPolicy:      pullPolicy,
+		recorder:        mgr.GetEventRecorderFor("import-controller"),
+		featureGates:    featuregates.NewFeatureGates(client),
+		installerLabels: installerLabels,
 	}
 	importController, err := controller.New("import-controller", mgr, controller.Options{
 		Reconciler: reconciler,
@@ -353,7 +349,7 @@ func (r *ImportReconciler) updatePvcFromPod(pvc *corev1.PersistentVolumeClaim, p
 
 	log.V(1).Info("Updating PVC from pod")
 	anno := pvc.GetAnnotations()
-	setConditionFromPodWithPrefix(anno, AnnRunningCondition, pod)
+	setAnnotationsFromPodWithPrefix(anno, pod, AnnRunningCondition)
 
 	scratchExitCode := false
 	if pod.Status.ContainerStatuses != nil &&
@@ -368,25 +364,11 @@ func (r *ImportReconciler) updatePvcFromPod(pvc *corev1.PersistentVolumeClaim, p
 			r.recorder.Event(pvc, corev1.EventTypeWarning, ErrImportFailedPVC, pod.Status.ContainerStatuses[0].LastTerminationState.Terminated.Message)
 		}
 	}
-	if pod.Status.ContainerStatuses != nil &&
-		pod.Status.ContainerStatuses[0].State.Terminated != nil &&
-		pod.Status.ContainerStatuses[0].State.Terminated.ExitCode == 0 {
-	}
-	if pod.Status.ContainerStatuses != nil &&
-		pod.Status.ContainerStatuses[0].State.Terminated != nil {
-		if getSource(pvc) == SourceVDDK {
-			r.saveVddkAnnotations(pvc, pod, log)
-		}
-	}
 
 	if anno[AnnCurrentCheckpoint] != "" {
 		anno[AnnCurrentPodID] = string(pod.ObjectMeta.UID)
 	}
 
-	if pod.Status.ContainerStatuses != nil {
-		anno[AnnPodRestarts] = strconv.Itoa(int(pod.Status.ContainerStatuses[0].RestartCount))
-	}
-
 	anno[AnnImportPod] = string(pod.Name)
 	if !scratchExitCode {
 		// No scratch exit code, update the phase based on the pod. If we do have scratch exit code we don't want to update the
@@ -425,40 +407,18 @@ func (r *ImportReconciler) updatePvcFromPod(pvc *corev1.PersistentVolumeClaim, p
 	if isPVCComplete(pvc) || scratchExitCode {
 		if !scratchExitCode {
 			r.recorder.Event(pvc, corev1.EventTypeNormal, ImportSucceededPVC, "Import Successful")
-			log.V(1).Info("Completed successfully, deleting POD", "pod.Name", pod.Name)
+			log.V(1).Info("Import completed successfully")
 		}
-		if err := r.client.Delete(context.TODO(), pod); IgnoreNotFound(err) != nil {
-			return err
+		if shouldDeletePod(pvc) {
+			log.V(1).Info("Deleting pod", "pod.Name", pod.Name)
+			if err := r.client.Delete(context.TODO(), pod); IgnoreNotFound(err) != nil {
+				return err
+			}
 		}
 	}
 	return nil
 }
 
-func (r *ImportReconciler) saveVddkAnnotations(pvc *corev1.PersistentVolumeClaim, pod *corev1.Pod, log logr.Logger) {
-	terminationMessage := pod.Status.ContainerStatuses[0].State.Terminated.Message
-	log.V(1).Info("Saving VDDK annotations from pod status message: ", "message", terminationMessage)
-
-	var terminationInfo string
-	matches := vddkInfoMatch.FindAllStringSubmatch(terminationMessage, -1)
-	for index, matchName := range vddkInfoMatch.SubexpNames() {
-		if matchName == "info" && len(matches) > 0 {
-			terminationInfo = matches[0][index]
-			break
-		}
-	}
-
-	var vddkInfo util.VddkInfo
-	err := json.Unmarshal([]byte(terminationInfo), &vddkInfo)
-	if err == nil {
-		if vddkInfo.Host != "" {
-			pvc.Annotations[AnnVddkHostConnection] = vddkInfo.Host
-		}
-		if vddkInfo.Version != "" {
-			pvc.Annotations[AnnVddkVersion] = vddkInfo.Version
-		}
-	}
-}
-
 func (r *ImportReconciler) updatePVC(pvc *corev1.PersistentVolumeClaim, log logr.Logger) error {
 	log.V(1).Info("Annotations are now", "pvc.anno", pvc.GetAnnotations())
 	if err := r.client.Update(context.TODO(), pvc); err != nil {
@@ -499,7 +459,7 @@ func (r *ImportReconciler) createImporterPod(pvc *corev1.PersistentVolumeClaim)
 		return err
 	}
 	// all checks passed, let's create the importer pod!
-	pod, err := createImporterPod(r.log, r.client, r.image, r.verbose, r.pullPolicy, podEnvVar, pvc, scratchPvcName, vddkImageName, getPriorityClass(pvc))
+	pod, err := createImporterPod(r.log, r.client, r.image, r.verbose, r.pullPolicy, podEnvVar, pvc, scratchPvcName, vddkImageName, getPriorityClass(pvc), r.installerLabels)
 
 	if err != nil {
 		return err
@@ -705,7 +665,7 @@ func (r *ImportReconciler) createScratchPvcForPod(pvc *corev1.PersistentVolumeCl
 
 		storageClassName := GetScratchPvcStorageClass(r.client, pvc)
 		// Scratch PVC doesn't exist yet, create it. Determine which storage class to use.
-		_, err = CreateScratchPersistentVolumeClaim(r.client, pvc, pod, scratchPVCName, storageClassName)
+		_, err = CreateScratchPersistentVolumeClaim(r.client, pvc, pod, scratchPVCName, storageClassName, r.installerLabels)
 		if err != nil {
 			return err
 		}
@@ -715,6 +675,7 @@ func (r *ImportReconciler) createScratchPvcForPod(pvc *corev1.PersistentVolumeCl
 	} else {
 		setBoundConditionFromPVC(anno, AnnBoundCondition, scratchPvc)
 	}
+	anno[AnnRequiresScratch] = "false"
 	return nil
 }
 
@@ -811,7 +772,7 @@ func createImportPodNameFromPvc(pvc *corev1.PersistentVolumeClaim) string {
 // createImporterPod creates and returns a pointer to a pod which is created based on the passed-in endpoint, secret
 // name, and pvc. A nil secret means the endpoint credentials are not passed to the
 // importer pod.
-func createImporterPod(log logr.Logger, client client.Client, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, vddkImageName *string, priorityClassName string) (*corev1.Pod, error) {
+func createImporterPod(log logr.Logger, client client.Client, image, verbose, pullPolicy string, podEnvVar *importPodEnvVar, pvc *corev1.PersistentVolumeClaim, scratchPvcName *string, vddkImageName *string, priorityClassName string, installerLabels map[string]string) (*corev1.Pod, error) {
 	podResourceRequirements, err := GetDefaultPodResourceRequirements(client)
 	if err != nil {
 		return nil, err
@@ -823,6 +784,7 @@ func createImporterPod(log logr.Logger, client client.Client, image, verbose, pu
 	}
 
 	pod := makeImporterPodSpec(pvc.Namespace, image, verbose, pullPolicy, podEnvVar, pvc, scratchPvcName, podResourceRequirements, workloadNodePlacement, vddkImageName, priorityClassName)
+	util.SetRecommendedLabels(pod, installerLabels, "cdi-controller")
 
 	if err := client.Create(context.TODO(), pod); err != nil {
 		return nil, err
@@ -992,7 +954,7 @@ func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar
 			MountPath: common.ImporterProxyCertDir,
 		}
 		pod.Spec.Containers[0].VolumeMounts = append(pod.Spec.Containers[0].VolumeMounts, vm)
-		pod.Spec.Volumes = append(pod.Spec.Volumes, createProxyConfigMapVolume(CertVolName, podEnvVar.certConfigMapProxy))
+		pod.Spec.Volumes = append(pod.Spec.Volumes, createProxyConfigMapVolume(ProxyCertVolName, podEnvVar.certConfigMapProxy))
 	}
 
 	if podEnvVar.contentType == string(cdiv1.DataVolumeKubeVirt) {
@@ -1009,7 +971,7 @@ func makeImporterPodSpec(namespace, image, verbose, pullPolicy string, podEnvVar
 
 func createProxyConfigMapVolume(certVolName, objRef string) corev1.Volume {
 	return corev1.Volume{
-		Name: CertVolName,
+		Name: certVolName,
 		VolumeSource: corev1.VolumeSource{
 			ConfigMap: &corev1.ConfigMapVolumeSource{
 				LocalObjectReference: corev1.LocalObjectReference{
