@@ -3,7 +3,10 @@ package controller
 import (
 	"context"
 	"crypto/rsa"
+	"encoding/json"
 	"fmt"
+	"regexp"
+	"strconv"
 	"strings"
 
 	featuregates "kubevirt.io/containerized-data-importer/pkg/feature-gates"
@@ -26,6 +29,7 @@ import (
 	cdiv1 "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1"
 	cdiv1utils "kubevirt.io/containerized-data-importer/pkg/apis/core/v1beta1/utils"
 	"kubevirt.io/containerized-data-importer/pkg/common"
+	"kubevirt.io/containerized-data-importer/pkg/util"
 	"kubevirt.io/containerized-data-importer/pkg/util/cert"
 	"kubevirt.io/containerized-data-importer/pkg/util/naming"
 	sdkapi "kubevirt.io/controller-lifecycle-operator-sdk/pkg/sdk/api"
@@ -65,6 +69,8 @@ const (
 	AnnPrePopulated = AnnAPIGroup + "/storage.prePopulated"
 	// AnnPriorityClassName is PVC annotation to indicate the priority class name for importer, cloner and uploader pod
 	AnnPriorityClassName = AnnAPIGroup + "/storage.pod.priorityclassname"
+	// AnnPodRetainAfterCompletion is PVC annotation for retaining transfer pods after completion)
+	AnnPodRetainAfterCompletion = AnnAPIGroup + "/storage.pod.retainAfterCompletion"
 
 	// AnnPreviousCheckpoint provides a const to indicate the previous snapshot for a multistage import
 	AnnPreviousCheckpoint = AnnAPIGroup + "/storage.checkpoint.previous"
@@ -145,6 +151,10 @@ const (
 	AnnPodSidecarInjectionDefault = "false"
 )
 
+var (
+	vddkInfoMatch = regexp.MustCompile(`((.*; )|^)VDDK: (?P<info>{.*})`)
+)
+
 func isCrossNamespaceClone(dv *cdiv1.DataVolume) bool {
 	if dv.Spec.Source.PVC == nil {
 		return false
@@ -225,6 +235,11 @@ func getRequestedImageSize(pvc *v1.PersistentVolumeClaim) (string, error) {
 	return pvcSize.String(), nil
 }
 
+// GetVolumeMode returns the volumeMode from PVC handling default empty value
+func GetVolumeMode(pvc *v1.PersistentVolumeClaim) v1.PersistentVolumeMode {
+	return getVolumeMode(pvc)
+}
+
 // returns the volumeMode from PVC handling default empty value
 func getVolumeMode(pvc *v1.PersistentVolumeClaim) v1.PersistentVolumeMode {
 	return resolveVolumeMode(pvc.Spec.VolumeMode)
@@ -288,8 +303,9 @@ func newScratchPersistentVolumeClaimSpec(pvc *v1.PersistentVolumeClaim, pod *v1.
 }
 
 // CreateScratchPersistentVolumeClaim creates and returns a pointer to a scratch PVC which is created based on the passed-in pvc and storage class name.
-func CreateScratchPersistentVolumeClaim(client client.Client, pvc *v1.PersistentVolumeClaim, pod *v1.Pod, name, storageClassName string) (*v1.PersistentVolumeClaim, error) {
+func CreateScratchPersistentVolumeClaim(client client.Client, pvc *v1.PersistentVolumeClaim, pod *v1.Pod, name, storageClassName string, installerLabels map[string]string) (*v1.PersistentVolumeClaim, error) {
 	scratchPvcSpec := newScratchPersistentVolumeClaimSpec(pvc, pod, name, storageClassName)
+	util.SetRecommendedLabels(scratchPvcSpec, installerLabels, "cdi-controller")
 	if err := client.Create(context.TODO(), scratchPvcSpec); err != nil {
 		if !k8serrors.IsAlreadyExists(err) {
 			return nil, errors.Wrap(err, "scratch PVC API create errored")
@@ -337,7 +353,6 @@ func GetDefaultStorageClass(client client.Client) (*storagev1.StorageClass, erro
 
 // GetFilesystemOverhead determines the filesystem overhead defined in CDIConfig for this PVC's volumeMode and storageClass.
 func GetFilesystemOverhead(client client.Client, pvc *v1.PersistentVolumeClaim) (cdiv1.Percent, error) {
-	klog.V(1).Info("GetFilesystemOverhead with PVC", pvc)
 	if getVolumeMode(pvc) != v1.PersistentVolumeFilesystem {
 		return "0", nil
 	}
@@ -546,28 +561,75 @@ func podSucceededFromPVC(pvc *v1.PersistentVolumeClaim) bool {
 	return (podPhaseFromPVC(pvc) == v1.PodSucceeded)
 }
 
-func setConditionFromPodWithPrefix(anno map[string]string, prefix string, pod *v1.Pod) {
-	if pod.Status.ContainerStatuses != nil {
-		if pod.Status.ContainerStatuses[0].State.Running != nil {
-			anno[prefix] = "true"
-			anno[prefix+".message"] = ""
-			anno[prefix+".reason"] = podRunningReason
-		} else {
-			anno[AnnRunningCondition] = "false"
-			if pod.Status.ContainerStatuses[0].State.Waiting != nil {
-				anno[prefix+".message"] = pod.Status.ContainerStatuses[0].State.Waiting.Message
-				anno[prefix+".reason"] = pod.Status.ContainerStatuses[0].State.Waiting.Reason
-			} else if pod.Status.ContainerStatuses[0].State.Terminated != nil {
-				anno[prefix+".message"] = pod.Status.ContainerStatuses[0].State.Terminated.Message
-				anno[prefix+".reason"] = pod.Status.ContainerStatuses[0].State.Terminated.Reason
-				if strings.Contains(pod.Status.ContainerStatuses[0].State.Terminated.Message, common.PreallocationApplied) {
-					anno[AnnPreallocationApplied] = "true"
-				}
+func setAnnotationsFromPodWithPrefix(anno map[string]string, pod *v1.Pod, prefix string) {
+	if pod == nil || pod.Status.ContainerStatuses == nil {
+		return
+	}
+	annPodRestarts, _ := strconv.Atoi(anno[AnnPodRestarts])
+	podRestarts := int(pod.Status.ContainerStatuses[0].RestartCount)
+	if podRestarts >= annPodRestarts {
+		anno[AnnPodRestarts] = strconv.Itoa(podRestarts)
+	}
+	setVddkAnnotations(anno, pod)
+	containerState := pod.Status.ContainerStatuses[0].State
+	if containerState.Running != nil {
+		anno[prefix] = "true"
+		anno[prefix+".message"] = ""
+		anno[prefix+".reason"] = podRunningReason
+	} else {
+		anno[AnnRunningCondition] = "false"
+		if containerState.Waiting != nil && containerState.Waiting.Reason != "CrashLoopBackOff" {
+			anno[prefix+".message"] = simplifyKnownMessage(containerState.Waiting.Message)
+			anno[prefix+".reason"] = containerState.Waiting.Reason
+		} else if containerState.Terminated != nil {
+			anno[prefix+".message"] = simplifyKnownMessage(containerState.Terminated.Message)
+			anno[prefix+".reason"] = containerState.Terminated.Reason
+			if strings.Contains(containerState.Terminated.Message, common.PreallocationApplied) {
+				anno[AnnPreallocationApplied] = "true"
 			}
 		}
 	}
 }
 
+func simplifyKnownMessage(msg string) string {
+	if strings.Contains(msg, "is larger than available size") ||
+		strings.Contains(msg, "no space left on device") ||
+		strings.Contains(msg, "file largest block is bigger than maxblock") {
+		return "DataVolume too small to contain image"
+	}
+
+	return msg
+}
+
+func setVddkAnnotations(anno map[string]string, pod *corev1.Pod) {
+	if pod.Status.ContainerStatuses[0].State.Terminated == nil {
+		return
+	}
+	terminationMessage := pod.Status.ContainerStatuses[0].State.Terminated.Message
+	klog.V(1).Info("Saving VDDK annotations from pod status message: ", "message", terminationMessage)
+
+	var terminationInfo string
+	matches := vddkInfoMatch.FindAllStringSubmatch(terminationMessage, -1)
+	for index, matchName := range vddkInfoMatch.SubexpNames() {
+		if matchName == "info" && len(matches) > 0 {
+			terminationInfo = matches[0][index]
+			break
+		}
+	}
+
+	var vddkInfo util.VddkInfo
+	err := json.Unmarshal([]byte(terminationInfo), &vddkInfo)
+	if err != nil {
+		return
+	}
+	if vddkInfo.Host != "" {
+		anno[AnnVddkHostConnection] = vddkInfo.Host
+	}
+	if vddkInfo.Version != "" {
+		anno[AnnVddkVersion] = vddkInfo.Version
+	}
+}
+
 func setBoundConditionFromPVC(anno map[string]string, prefix string, pvc *v1.PersistentVolumeClaim) {
 	switch pvc.Status.Phase {
 	case v1.ClaimBound:
@@ -718,7 +780,9 @@ func GetPreallocation(client client.Client, dataVolume *cdiv1.DataVolume) bool {
 // GetClusterWideProxy returns the OpenShift cluster wide proxy object
 func GetClusterWideProxy(r client.Client) (*ocpconfigv1.Proxy, error) {
 	clusterWideProxy := &ocpconfigv1.Proxy{}
-	if err := r.Get(context.TODO(), types.NamespacedName{Name: ClusterWideProxyName}, clusterWideProxy); err != nil {
+	// Ignore both no CRD found (IgnoreIsNoMatch) and the object itself not existing IsNotFound because we want to skip if not
+	// in Open Shift.
+	if err := r.Get(context.TODO(), types.NamespacedName{Name: ClusterWideProxyName}, clusterWideProxy); IgnoreIsNoMatchError(err) != nil && !k8serrors.IsNotFound(err) {
 		return nil, err
 	}
 	return clusterWideProxy, nil
@@ -763,6 +827,10 @@ func getPriorityClass(pvc *v1.PersistentVolumeClaim) string {
 	return anno[AnnPriorityClassName]
 }
 
+func shouldDeletePod(pvc *v1.PersistentVolumeClaim) bool {
+	return pvc.GetAnnotations()[AnnPodRetainAfterCompletion] != "true" || pvc.GetAnnotations()[AnnRequiresScratch] == "true" || pvc.DeletionTimestamp != nil
+}
+
 // AddFinalizer adds a finalizer to a resource
 func AddFinalizer(obj metav1.Object, name string) {
 	if HasFinalizer(obj, name) {
