@@ -129,7 +129,6 @@ func NewDataProcessor(dataSource DataSourceInterface, dataFile, dataDir, scratch
 	if isVddk {
 		needsDataCleanup = !vddkSource.IsDeltaCopy()
 	}
-
 	dp := &DataProcessor{
 		currentPhase:       ProcessingPhaseInfo,
 		source:             dataSource,
@@ -314,6 +313,11 @@ func ResizeImage(dataFile, imageSize string, totalTargetSpace int64, preallocati
 			klog.V(1).Infof("No need to resize image. Requested size: %s, Image size: %d.\n", imageSize, info.VirtualSize)
 			return nil
 		}
+		// Check if calculated size is < imageSize, and return error if so.
+		if currentImageSizeQuantity.Cmp(minSizeQuantity) == 1 {
+			klog.V(1).Infof("Calculated new size is < than current size, not resizing: requested size %s, virtual size: %d.\n", minSizeQuantity.String(), info.VirtualSize)
+			return nil
+		}
 		klog.V(1).Infof("Expanding image size to: %s\n", minSizeQuantity.String())
 		return qemuOperations.Resize(dataFile, minSizeQuantity, preallocation)
 	}
@@ -357,14 +361,5 @@ func (dp *DataProcessor) PreallocationApplied() bool {
 }
 
 func (dp *DataProcessor) getUsableSpace() int64 {
-	return GetUsableSpace(dp.filesystemOverhead, dp.availableSpace)
-}
-
-// GetUsableSpace calculates space to use taking file system overhead into account
-func GetUsableSpace(filesystemOverhead float64, availableSpace int64) int64 {
-	spaceWithOverhead := int64((1 - filesystemOverhead) * float64(availableSpace))
-	// qemu-img will round up, making us use more than the usable space.
-	// This later conflicts with image size validation.
-	qemuImgCorrection := util.RoundDown(spaceWithOverhead, util.DefaultAlignBlockSize)
-	return qemuImgCorrection
+	return util.GetUsableSpace(dp.filesystemOverhead, dp.availableSpace)
 }
